@@ -5,6 +5,7 @@ import pathlib
 import time
 import typing
 import webbrowser
+from warnings import warn
 
 import requests
 from bs4 import BeautifulSoup as Soup
@@ -12,6 +13,7 @@ from bs4 import BeautifulSoup as Soup
 from .data import HEADERS
 
 T = typing.TypeVar("T")
+U = typing.TypeVar("U")
 
 try:
     from rich import print, progress
@@ -27,9 +29,9 @@ except ImportError:
         print(msg)
         time.sleep(secs)
 
-    def work(msg: str, worker: typing.Callable[[], T]) -> T:
+    def work(msg: str, worker: typing.Callable[[U], T], data: U) -> T:
         print(msg)
-        return worker()
+        return worker(data)
 
 else:
     RED = "[red]"
@@ -48,7 +50,7 @@ else:
         ):
             time.sleep(0.1)
 
-    def _rich_work(msg: str, worker: typing.Callable[[], T]) -> T:
+    def _rich_work(msg: str, worker: typing.Callable[[U], T], data: U) -> T:
         with progress.Progress(
             progress.TextColumn("{task.description}"),
             progress.SpinnerColumn(),
@@ -56,7 +58,7 @@ else:
             transient=True,
         ) as bar:
             task = bar.add_task(msg)
-            val = worker()
+            val = worker(data)
             bar.advance(task)
             return val
 
@@ -184,7 +186,8 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
             "has already been solved.\nThe solution was: "
             f"{BLUE}{solution}{RESET}"
         )
-        if match := RANK.search(solutions[part_][solution]):
+        match = RANK.search(solutions[part_][solution])
+        if match:
             _pretty_print(match.group(0))
         return
 
@@ -269,7 +272,10 @@ def submit_25(year: str):
 
 
 def lazy_submit(
-    day: int, solution: typing.Callable[[], typing.Any], year: int = DEFAULT_YEAR
+    day: int,
+    solution: typing.Callable[[U], typing.Any],
+    data: U,
+    year: int = DEFAULT_YEAR,
 ) -> None:
     """Run the function only if we haven't seen a solution.
 
@@ -295,13 +301,135 @@ def lazy_submit(
             "has already been solved.\nThe solution was: "
             f"{BLUE}{solution_}{RESET}"
         )
-        if match := RANK.search(solutions[str(part)][solution_]):
+        match = RANK.search(solutions[str(part)][solution_])
+        if match:
             _pretty_print(match.group(0))
-    elif (
-        answer := work(
+    else:
+        answer = work(
             f"{YELLOW}Running part"
             f" {RESET}{BLUE}{part}{RESET}{YELLOW} solution...{RESET}",
             solution,
+            data,
         )
-    ) is not None:
-        submit(day, part, answer, year)
+        if answer is not None:
+            submit(day, part, answer, year)
+
+
+def get_sample_input(
+    day: int, year: int = DEFAULT_YEAR
+) -> typing.Optional[typing.Tuple[str, str]]:
+    """Retrieves the example input and answer for the corresponding AOC challenge."""
+    resp = requests.post(
+        url=URL.format(day=day, year=year), cookies=get_cookie(), headers=HEADERS
+    )
+    soup = Soup(resp.text, "html.parser")
+
+    # Find the example test input for that day.
+    for possible_test_input in soup.find_all("pre"):
+        preceding_text = (
+            possible_test_input.previous_element.previous_element.text.lower()
+        )
+        if (
+            "for example" in preceding_text or "consider" in preceding_text
+        ) and ":" in preceding_text:
+            test_input = possible_test_input.text.strip()
+            break
+        elif len(possible_test_input.text.split("\n")) > 1:
+            test_input = possible_test_input.text.strip()
+            break
+    else:
+        return None
+
+    # Attempt to retrieve answer to said example data.
+    current_part = soup.find_all("article")[-1]
+    last_sentence = current_part.find_all("p")[-2]
+
+    try:
+        answer = last_sentence.find_all("code")[-1]
+    except IndexError:
+        return None
+    if not answer.em:
+        try:
+            answer = last_sentence.find_all("em")[-1]
+        except IndexError:
+            pass
+
+    answer = answer.text.strip().split()[-1]
+
+    return test_input, answer
+
+
+def test(
+    day: int, part: int, answer: str, expected_answer: str, year: int = DEFAULT_YEAR
+) -> None:
+    day_ = str(day)
+    year_ = str(year)
+    part_ = str(part)
+
+    testing_dir = DATA_DIR / year_ / day_
+    _make(testing_dir)
+
+    # Load cached tests
+    tests = testing_dir / "tests.json"
+    if tests.exists() and tests.read_text():
+        with tests.open() as f:
+            test_data = json.load(f)
+    else:
+        test_data = {"1": [], "2": []}
+
+    test_data[part_].append({"answer": answer, "expected_answer": expected_answer})
+
+    with tests.open("w") as f:
+        json.dump(test_data, f)
+
+    assert answer == expected_answer, (
+        f"The expected answer for the example test input was {expected_answer} but"
+        f" your answer was {answer}."
+    )
+
+
+def lazy_test(
+    day: int,
+    parse: typing.Callable[[str], T],
+    solution: typing.Callable[[T], typing.Any],
+    year: int = DEFAULT_YEAR,
+    test_data: typing.Optional[typing.Tuple[str, typing.Any]] = None,
+) -> None:
+    """Test the function with AOC's example data only if we haven't tested it already.
+
+    Solution is expected to be named 'part_one' or 'part_two'
+    """
+    part = 1 if solution.__name__ == "part_one" else 2
+    testing_dir = DATA_DIR / str(year) / str(day)
+    testing_file = testing_dir / "tests.json"
+
+    # Check if the tests have ran for the specific part yet
+    if not testing_file.exists() or (
+        (testing_dir / "1.solution").exists() and part != 1
+    ):
+        if test_data is None:  # No test data passed (most common)
+            test_data = get_sample_input(day, year)
+            if test_data is None:  # No test data scraped (uncommon)
+                warn(
+                    f"An issue occurred while fetching test data for {year} day"
+                    f" {day} part {part}. You may either ignore this message, or pass"
+                    " custom test data to lazy_test.",
+                    RuntimeWarning,
+                )
+                return
+        test_input, test_answer = test_data
+
+        answer = work(
+            f"{YELLOW}Running the test for part"
+            f" {RESET}{BLUE}{part}{RESET}{YELLOW} solution...{RESET}",
+            solution,
+            parse(test_input),
+        )
+        if answer is not None:
+            test(day, part, str(answer).strip(), str(test_answer).strip(), year)
+            print(
+                f"{GREEN}Test for part {RESET}{YELLOW}{part}{YELLOW}{RESET} succeeded!"
+                f" {RESET}Your answer for part 2 with the test data was:"
+                f" {GREEN}{answer}{GREEN}{RESET} and the expected answer with the test"
+                f" data was also: {GREEN}{test_answer}{GREEN}{RESET}"
+            )
