@@ -26,7 +26,7 @@ from aoc_helper.types import (
 
 T = typing.TypeVar("T")
 U = typing.TypeVar("U")
-GenericU = typing.TypeVar("GenericU", bound=typing.Generic)
+GenericU = typing.Generic[T]
 P = ParamSpec("P")
 
 
@@ -36,11 +36,55 @@ MaybeIterator = typing.Union[T, typing.Iterable["MaybeIterator[T]"]]
 def extract_ints(raw: str) -> "list[int]":
     """Utility function to extract all integers from some string.
 
-    Many inputs can be directly parsed with this function.
+    Some inputs can be directly parsed with this function.
     """
     return list(map(int, re.findall(r"((?:-|\+)?\d+)", raw)))
 
 
+def extract_uints(raw: str) -> "list[int]":
+    """Utility function to extract all integers from some string.
+
+    Minus signs will be *ignored*; the output integers will all be positive.
+
+    Some inputs can be directly parsed with this function.
+    """
+    return list(map(int, re.findall(r"(\d+)", raw)))
+
+
+def _range_from_match(range: typing.Tuple[str, str]) -> builtins.range:
+    if range[1]:
+        return builtins.range(int(range[0]), int(range[1]))
+    else:
+        return builtins.range(int(range[0]), int(range[0]))
+
+
+def _irange_from_match(range: typing.Tuple[str, str]) -> builtins.range:
+    if range[1]:
+        return builtins.range(int(range[0]), int(range[1]) + 1)
+    else:
+        return builtins.range(int(range[0]), int(range[0]) + 1)
+
+
+def extract_ranges(raw: str) -> "list[builtins.range]":
+    """Utility function to extract all ranges from some string.
+
+    Ranges are interpreted as `start-stop` and are not inclusive.
+
+    Some inputs can be directly parsed with this function.
+    """
+    return list(map(_range_from_match, re.findall(r"(\d+)(?:-(\d+))?", raw)))
+
+
+def extract_iranges(raw: str) -> "list[builtins.range]":
+    """Utility function to extract all ranges from some string.
+
+    Ranges are interpreted as `start-stop` and are inclusive.
+
+    Some inputs can be directly parsed with this function.
+    """
+    return list(map(_irange_from_match, re.findall(r"(\d+)(?:-(\d+))?", raw)))
+
+
 def chunk(
     iterable: typing.Iterable[T], chunk_size: int
 ) -> typing.Iterable[tuple[T, ...]]:
@@ -124,6 +168,33 @@ class list(UserList, typing.Generic[T]):
             if pred(i):
                 return i
 
+    def any(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if any element satisfies the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if any element is truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return any(pred(item) for item in self)
+
+    def all(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if all elements satisfy the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if all elements are truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return all(pred(item) for item in self)
+
+    def none(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if no element satisfies the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if no element is truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return not any(pred(item) for item in self)
+
     def windowed(self, window_size: int) -> "list[typing.Tuple[T, ...]]":
         """Return an list containing the elements of this list in
         a sliding window of size window_size. If there are not enough elements
@@ -389,7 +460,7 @@ class list(UserList, typing.Generic[T]):
                 else list(item).flat(True)
                 if isinstance(item, (builtins.list, list))
                 else item
-            )  # type: ignore
+            )
         )
 
     def enumerated(self, start: int = 0) -> "list[typing.Tuple[int, T]]":
@@ -418,7 +489,7 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
     def __init__(self, it: typing.Iterable[T]) -> None:
         self.it = builtins.iter(it)
 
-    def __iter__(self) -> typing.Iterable[T]:
+    def __iter__(self) -> typing.Iterator[T]:
         return self.it.__iter__()
 
     def __next__(self) -> T:
@@ -439,15 +510,12 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
         return iter(self.map(lambda i: iter(i).map(func)))
 
     def filter(
-        self, pred: typing.Union[typing.Callable[[T], bool], T, None] = None
+        self, pred: typing.Union[typing.Callable[[T], bool], T] = bool
     ) -> "iter[T]":
         """Return an iterator containing only the elements for which pred
         returns True.
 
-        If pred is None, return an iterator containing only elements that are
-        truthy.
-
-        If pred is a T (and T is not a callable or None), return an iterator
+        If pred is a T (and T is not callable), return an iterator
         containing only elements that compare equal to pred.
         """
         if not callable(pred) and pred is not None:
@@ -470,10 +538,37 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
             pred = lambda i: bool(i)
         elif not callable(pred):
             pred = (lambda j: lambda i: i == j)(pred)
-        for i in self:  # type: ignore
+        for i in self:
             if pred(i):
                 return i
 
+    def any(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if any element satisfies the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if any element is truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return any(pred(item) for item in self)
+
+    def all(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if all elements satisfy the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if all elements are truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return all(pred(item) for item in self)
+
+    def none(self, pred: typing.Union[typing.Callable[[T], bool], T] = bool) -> bool:
+        """Consume this iterator and return True if no element satisfies the
+        given predicate. The default predicate is bool; therefore by default this
+        method returns True if no element is truthy.
+        """
+        if not callable(pred):
+            pred = (lambda j: lambda i: i == j)(pred)
+        return not any(pred(item) for item in self)
+
     @typing.overload
     def reduce(self, func: typing.Callable[[T, T], T]) -> T:
         ...
@@ -518,7 +613,7 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
 
     def foreach(self, func: typing.Callable[[T], typing.Any]) -> None:
         """Run func on every value in this iterator, immediately."""
-        for el in self:  # type: ignore
+        for el in self:
             func(el)
 
     def chunk(self, n: int) -> "iter[typing.Tuple[T, ...]]":
@@ -538,8 +633,8 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
     def _window(
         self, window_size: int
     ) -> typing.Generator[typing.Tuple[T, ...], None, None]:
-        elements = deque()
-        for _ in range(window_size):  # type: ignore
+        elements: typing.Deque[T] = deque()
+        for _ in range(window_size):
             try:
                 elements.append(self.next())
             except StopIteration:
@@ -547,7 +642,7 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
 
         yield tuple(elements)
 
-        for el in self:  # type: ignore
+        for el in self:
             elements.popleft()
             elements.append(el)
             yield tuple(elements)
@@ -614,8 +709,8 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
     def collect(self) -> list[T]:
         ...
 
-    @typing.overload
-    def collect(self, collection_type: typing.Type[GenericU]) -> "GenericU[T]":
+    @typing.overload  # TODO: why doesn't this work?
+    def collect(self, collection_type: typing.Type[U]) -> "U[T]":
         ...
 
     def collect(self, collection_type=None):
@@ -806,7 +901,7 @@ class iter(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
             return iter(item for iterator in self for item in iterator)  # type: ignore
         return iter(
             item
-            for iterator in self  # type: ignore
+            for iterator in self
             for item in (
                 iterator.flatten(True)
                 if isinstance(iterator, iter)
@@ -1015,7 +1110,7 @@ def decode_text(dots: typing.List[typing.List[bool]]) -> str:
     return "".join(out)
 
 
-def _default_classifier(char: str) -> int:
+def _default_classifier(char: str, /) -> int:
     if char in "0123456789":
         return int(char)
     elif char in ".#":
@@ -1041,7 +1136,7 @@ class Grid(typing.Generic[T]):
         ...
 
     @classmethod
-    def from_string(cls, data: str, classify=_default_classifier):  # type: ignore
+    def from_string(cls, data: str, classify=_default_classifier):
         """Create a grid from a string (e.g. a puzzle input).
 
         Can take a classifier to use a custom classification. The default will
@@ -1059,7 +1154,7 @@ class Grid(typing.Generic[T]):
 
         start defaults to the top left, and end defaults to the bottom right.
         """
-        to_visit = []
+        to_visit: typing.List[typing.Tuple[int, typing.Tuple[int, int]]] = []
         heappush(to_visit, (0, start or (0, 0)))
         visited = set()
         if end is None:
@@ -1184,3 +1279,19 @@ class PrioQueue(typing.Generic[T], typing.Iterator[T], typing.Iterable[T]):
 
     def push(self, val: T) -> None:
         heappush(self._data, val)
+
+
+def chinese_remainder_theorem(
+    moduli: typing.List[int], residues: typing.List[int]
+) -> int:
+    from math import prod
+
+    N = prod(moduli)
+
+    return (
+        sum(
+            (div := (N // modulus)) * pow(div, -1, modulus) * residue
+            for modulus, residue in zip(moduli, residues)
+        )
+        % N
+    )
